import sys
import subprocess

from babel import Locale as BabelLocale
from babel.core import UnknownLocaleError as BabelUnknownLocaleError


def get_locale_name_babel(code):
    language, country = code[:-(len('.utf8'))].split('_')
    try:
        locale = BabelLocale(language, country)
    except BabelUnknownLocaleError:
        sys.stderr.write("babel raised BabelUnknownLocaleError for %s\n" % code)
        return u"Unknown (Unknown)"
    if locale.display_name is None:
        sys.stderr.write("babel got display_name = None for %s\n" % code)
        return u"Unknown (Unknown)"
    return locale.display_name

def read_all_codes():
    fdp = subprocess.Popen(['locale', '-a'], stdout=subprocess.PIPE)
    lines = fdp.stdout.read().split('\n')
    codes = []

    for line in lines:
        if line.endswith('utf8'):
            codes.append(line)

    #FIXME: This is a temporary workaround for locales that are essential to
    # OLPC, but are not in Glibc yet.
    for code in ['ht_HT.utf8', 'fa_AF.utf8', 'ps_AF.utf8']:
        codes.append(code)

    return codes

if __name__ == '__main__':
    available_codes = read_all_codes()

    names_babel = {}
    for code in available_codes:
        locale_name = get_locale_name_babel(code)
        names_babel[code] = locale_name
