import sys
import gettext
import subprocess


def get_locale_name_gettext(code, language_name, country_name,
                            lang_iso='iso_639'):
    language, country = code.split('_')

    language_translation = None
    try:
        translation = gettext.translation(lang_iso, languages=[language]) 
        language_translation = translation.ugettext(language_name)
    except IOError:
        sys.stderr.write("ugettext raised IOError for %s using %s\n" % (code, lang_iso))
        language_translation = "Unknown"
    except UnicodeError:
        sys.stderr.write("ugettext raised UnicodeError for %s\n" % code)
        language_translation = "Unknown"

    country_translation = None
    try:
        translation = gettext.translation('iso_3166', languages=[language])
        country_translation = translation.ugettext(country_name)
    except IOError:
        sys.stderr.write("ugettext raised IOError for %s using %s\n" % (code, lang_iso))
        country_translation = "Unknown"
    except UnicodeError:
        sys.stderr.write("ugettext raised UnicodeError for %s\n" % code)
        country_translation = "Unknown"

    return "%s (%s)" % (language_translation, country_translation)

def read_all_languages():
    fdp = subprocess.Popen(['locale', '-av'], stdout=subprocess.PIPE)
    lines = fdp.stdout.read().split('\n')
    locales = []

    for line in lines:
        if line.find('locale:') != -1:
            locale = line.split()[1]
        elif line.find('language |') != -1:
            lang = line.lstrip('language |')
        elif line.find('territory |') != -1:
            territory = line.lstrip('territory |')
            if locale.endswith('utf8') and len(lang):
                locales.append((lang, territory, locale))

    #FIXME: This is a temporary workaround for locales that are essential to
    # OLPC, but are not in Glibc yet.
    locales.append(('Kreyol', 'Haiti', 'ht_HT.utf8'))
    locales.append(('Dari', 'Afghanistan', 'fa_AF.utf8'))
    locales.append(('Pashto', 'Afghanistan', 'ps_AF.utf8'))

    locales.sort()
    return locales

if __name__ == '__main__':
    available_locales = read_all_languages()

    names_gettext_iso_639 = {}
    for language, country, code in available_locales:
        locale_name = get_locale_name_gettext(code, language, country,
                                              lang_iso='iso_639')
        names_gettext_iso_639[code] = locale_name
